
#include "inc.h"

#include "minix.h"
#include "irq_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "gbfs.h"
#include "lib.h"

#include "screen.h"

//---------------------------------------------------------------------------
//lib.c
extern ST_TIMER Timer;
extern ST_FONT  Font;

//---------------------------------------------------------------------------
ST_SCREEN_IMG  ScreenImg  EWRAM_DATA;
ST_CURSOR      Cursor;

u32 dummy ALIGN(4);
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgInit()
{
	LibMode3SetScreenBuffer((u16*)&ScreenImg.buf);

	dummy = 0;
	ScreenImgClear();
	ScreenImgUpdate();
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgClear()
{
	ScreenImg.x = 0;
	ScreenImg.y = 0;

	CpuSet((u32)&dummy, (u32)ScreenImg.buf, ((240*160*2) / 4) | CPUSET_32BIT | CPUSET_SRC_FIX);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgUpdate()
{
	WaitForVsync();
	CpuSet((u32)ScreenImg.buf, (u32)VideoBuffer, ((240*160*2) / 4) | CPUSET_32BIT);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgDrawStr(u8* str, u16 len)
{
	ST_FONT* f = &Font;
	ST_SCREEN_IMG* p = &ScreenImg;

	u16 i;
	for(i=0; i<len; i++)
	{
		if(p->x + f->half.cx >= SCREEN_CX)
		{
			p->x  = 0;
			p->y += f->half.cy;

			if(p->y + f->half.cy >= SCREEN_CY)
			{
				ScreenImgClear();
//				p->y = 0;	//TODO
			}
		}

		ScreenImgDrawChr(p->x, p->y, str[i]);
		p->x += f->half.cx;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgDrawChr(u16 x, u16 y, u8 chr)
{
	Mode3DrawFontHalfChr(x, y, chr, COLOR_WHITE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgNewLine()
{
	ST_FONT* f = &Font;
	ST_SCREEN_IMG* p = &ScreenImg;

	if(p->x == 0)
	{
		return;
	}

	p->x  = 0;
	p->y += f->half.cy;

	if(p->y + f->half.cy >= SCREEN_CY)
	{
		ScreenImgClear();
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgDrawNum(s32 var)
{
	u8  buf[32];
	u16 len;

	_Sprintf((char*)buf, "%d", var);
	len = _Strlen(buf);

	ScreenImgDrawStr(buf, len);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgSkipTab()
{
	ST_FONT* f = &Font;
	ST_SCREEN_IMG* p = &ScreenImg;

	u16 space = SCREEN_IMG_TAB_CNT - ( _Div(p->x, f->half.cx) % SCREEN_IMG_TAB_CNT);

	p->x += space * f->half.cx;
}





//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurInit()
{
	ST_CURSOR* p = &Cursor;

	p->x         = 0;
	p->y         = SCREEN_CY;
	p->sprNo     = CURSOR_SPRITE_NO;
	p->tileNo    = CURSOR_TILE_NO_ARROW;
	p->blinkFlag = 0x00;

	SpriteSetSize(p->sprNo, SP_SIZE_16, SP_SQUARE, SP_COLOR_16);
	SpriteSetChr (p->sprNo, p->tileNo);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurMove(u16 x, u16 y)
{
	ST_CURSOR* p = &Cursor;

	p->x = x;
	p->y = y;

	SpriteMove(p->sprNo, p->x, p->y);
}
